#!/usr/bin/env python

##############################################
# The MIT License (MIT)
# Copyright (c) 2016 Kevin Walchko
# see LICENSE for full details
##############################################
# ----------------------------
# Simple tool to change the id number of a servo
#

from __future__ import print_function, division
from pyxl320 import ServoSerial
from pyxl320 import Packet
from pyxl320 import xl320
# from pyxl320 import DummySerial
import argparse


def handleArgs():
	parser = argparse.ArgumentParser(description='Resets servo(s) to factory defaults')
	parser.add_argument('-a', '--all', help='reset all servos to defaults', action='store_true')
	parser.add_argument('-i', '--id', help='servo id', type=int, default=1)
	parser.add_argument('-p', '--port', help='serial port or \'dummy\' for testing, default is \'/dev/serial0\'', type=str, default='/dev/serial0')

	args = vars(parser.parse_args())
	return args


def main():
	args = handleArgs()

	port = args['port']

	if args['all']:
		ID = xl320.XL320_BROADCAST_ADDR
	else:
		ID = args['id']

	if port.lower() == 'dummy':
		ser = ServoSerial(port=port, fake=True)
	else:
		ser = ServoSerial(port=port)
	ser.open()

	pkt = Packet.makeRebootPacket(ID)
	# print('pkt', pkt)

	ser.write(pkt)
	gramar = 'is'
	if ID == xl320.XL320_BROADCAST_ADDR:
		ID = 'all'
		gramar = 'are'
	print('Servo[{}] {} rebooting'.format(ID, gramar))


if __name__ == '__main__':
	main()



#!/usr/bin/env python

##############################################
# The MIT License (MIT)
# Copyright (c) 2016 Kevin Walchko
# see LICENSE for full details
##############################################
# Send ping commands to all servos

from __future__ import print_function
from pyxl320.Packet import makePingPacket, packetToDict
from pyxl320 import ServoSerial
# from pyxl320 import DummySerial
from pyxl320 import utils
import argparse
import time
from pyxl320 import xl320


def sweep(port, rate, ID, retry=3):
	"""
	Sends a ping packet to ID's from 0 to maximum and prints out any returned
	messages.

	Actually send a broadcast and will retry (resend) the ping 3 times ...
	"""
	if port == 'dummy':
		s = ServoSerial(port, rate, fake=True)
	else:
		s = ServoSerial(port, rate)

	if ID < 0:
		ID = xl320.XL320_BROADCAST_ADDR

	s.open()
	pkt = makePingPacket(ID)
	print('ping', pkt)
	s.write(pkt)

	# as more servos add up, I might need to increase the cnt number???
	for cnt in range(retry):
		ans = s.read()

		if ans:
			for pkt in ans:
				servo = packetToDict(pkt)
				utils.prettyPrintPacket(servo)
				print('raw pkt: {}'.format(pkt))
		else:
			print('Try {}: no servos found'.format(cnt))

		time.sleep(0.1)

	s.close()


def handleArgs():
	parser = argparse.ArgumentParser(description='ping servos')
	# parser.add_argument('-m', '--max', help='max id', type=int, default=253)
	parser.add_argument('-r', '--rate', help='servo baud rate', type=int, default=1000000)
	parser.add_argument('-i', '--id', help='ping servo ID', type=int, default=-1)
	parser.add_argument('-p', '--port', help='serial port name, set to "dummy" for testing', type=str, default='/dev/serial0')

	args = vars(parser.parse_args())
	return args


if __name__ == '__main__':
	args = handleArgs()
	print('Finding all servos:')
	sweep(port=args['port'], rate=args['rate'], ID=args['id'])





###############################################
# The MIT License (MIT)
# Copyright (c) 2016 Kevin Walchko
# see LICENSE for full details
##############################################

from __future__ import division
from __future__ import print_function
from . import xl320

"""
A bunch of packet constructors, file name follows make* and returns a complete
packet (array of ints). The serial class will turn it into an array of bytes using
bytes(bytearray(pkt)) before sending it.

Packet format:

[0xFF, 0xFF, 0xFD, 0x00, ID, LEN_L, LEN_H, INST, PARAM 1, PARAM 2, ..., PARAM N, CRC_L, CRC_H]

Header: 0xFF, 0xFF, 0xFD
Reserved byte: 0x00
Servo ID: ID
Packet Length: Len_l, len_h
Instruction: INST
Parameters: Param 1 ... Param N
CRC: crc_l, crc_h

This code uses:
http://support.robotis.com/en/product/dynamixel_pro/communication/instruction_status_packet.htm
"""
crc_table = [
	0x0000, 0x8005, 0x800F, 0x000A, 0x801B, 0x001E, 0x0014, 0x8011,
	0x8033, 0x0036, 0x003C, 0x8039, 0x0028, 0x802D, 0x8027, 0x0022,
	0x8063, 0x0066, 0x006C, 0x8069, 0x0078, 0x807D, 0x8077, 0x0072,
	0x0050, 0x8055, 0x805F, 0x005A, 0x804B, 0x004E, 0x0044, 0x8041,
	0x80C3, 0x00C6, 0x00CC, 0x80C9, 0x00D8, 0x80DD, 0x80D7, 0x00D2,
	0x00F0, 0x80F5, 0x80FF, 0x00FA, 0x80EB, 0x00EE, 0x00E4, 0x80E1,
	0x00A0, 0x80A5, 0x80AF, 0x00AA, 0x80BB, 0x00BE, 0x00B4, 0x80B1,
	0x8093, 0x0096, 0x009C, 0x8099, 0x0088, 0x808D, 0x8087, 0x0082,
	0x8183, 0x0186, 0x018C, 0x8189, 0x0198, 0x819D, 0x8197, 0x0192,
	0x01B0, 0x81B5, 0x81BF, 0x01BA, 0x81AB, 0x01AE, 0x01A4, 0x81A1,
	0x01E0, 0x81E5, 0x81EF, 0x01EA, 0x81FB, 0x01FE, 0x01F4, 0x81F1,
	0x81D3, 0x01D6, 0x01DC, 0x81D9, 0x01C8, 0x81CD, 0x81C7, 0x01C2,
	0x0140, 0x8145, 0x814F, 0x014A, 0x815B, 0x015E, 0x0154, 0x8151,
	0x8173, 0x0176, 0x017C, 0x8179, 0x0168, 0x816D, 0x8167, 0x0162,
	0x8123, 0x0126, 0x012C, 0x8129, 0x0138, 0x813D, 0x8137, 0x0132,
	0x0110, 0x8115, 0x811F, 0x011A, 0x810B, 0x010E, 0x0104, 0x8101,
	0x8303, 0x0306, 0x030C, 0x8309, 0x0318, 0x831D, 0x8317, 0x0312,
	0x0330, 0x8335, 0x833F, 0x033A, 0x832B, 0x032E, 0x0324, 0x8321,
	0x0360, 0x8365, 0x836F, 0x036A, 0x837B, 0x037E, 0x0374, 0x8371,
	0x8353, 0x0356, 0x035C, 0x8359, 0x0348, 0x834D, 0x8347, 0x0342,
	0x03C0, 0x83C5, 0x83CF, 0x03CA, 0x83DB, 0x03DE, 0x03D4, 0x83D1,
	0x83F3, 0x03F6, 0x03FC, 0x83F9, 0x03E8, 0x83ED, 0x83E7, 0x03E2,
	0x83A3, 0x03A6, 0x03AC, 0x83A9, 0x03B8, 0x83BD, 0x83B7, 0x03B2,
	0x0390, 0x8395, 0x839F, 0x039A, 0x838B, 0x038E, 0x0384, 0x8381,
	0x0280, 0x8285, 0x828F, 0x028A, 0x829B, 0x029E, 0x0294, 0x8291,
	0x82B3, 0x02B6, 0x02BC, 0x82B9, 0x02A8, 0x82AD, 0x82A7, 0x02A2,
	0x82E3, 0x02E6, 0x02EC, 0x82E9, 0x02F8, 0x82FD, 0x82F7, 0x02F2,
	0x02D0, 0x82D5, 0x82DF, 0x02DA, 0x82CB, 0x02CE, 0x02C4, 0x82C1,
	0x8243, 0x0246, 0x024C, 0x8249, 0x0258, 0x825D, 0x8257, 0x0252,
	0x0270, 0x8275, 0x827F, 0x027A, 0x826B, 0x026E, 0x0264, 0x8261,
	0x0220, 0x8225, 0x822F, 0x022A, 0x823B, 0x023E, 0x0234, 0x8231,
	0x8213, 0x0216, 0x021C, 0x8219, 0x0208, 0x820D, 0x8207, 0x0202
]


def crc16(data_blk):
	"""
	Calculate crc

	in: data_blk - entire packet except last 2 crc bytes
	out: crc_accum - 16 word
	"""
	data_blk_size = len(data_blk)
	crc_accum = 0
	for j in range(data_blk_size):
		i = ((crc_accum >> 8) ^ data_blk[j]) & 0xFF
		crc_accum = ((crc_accum << 8) ^ crc_table[i])
		crc_accum &= 0xffff  # keep to 16 bits

	return crc_accum


# util function?
def le(h):
	"""
	Little-endian, takes a 16b number and returns an array arrange in little
	endian or [low_byte, high_byte].
	"""

	h &= 0xffff  # make sure it is 16 bits
	return [h & 0xff, h >> 8]


def word(l, h):
	"""
	Given a low and high bit, converts the number back into a word.
	"""
	return (h << 8) + l


def angleToInt(angle):
	"""
	Converts an angle to an integer the servo understands
	"""
	return int(angle/300.0*1023)


def makePacket(ID, instr, reg=None, params=None):
	"""
	This makes a generic packet.

	TODO: look a struct ... does that add value using it?

	0xFF, 0xFF, 0xFD, 0x00, ID, LEN_L, LEN_H, INST, PARAM 1, PARAM 2, ..., PARAM N, CRC_L, CRC_H]
	in:
		ID - servo id
		instr - instruction
		reg - register
		params - instruction parameter values
	out: packet
	"""
	pkt = []
	pkt += [0xFF, 0xFF, 0xFD]  # header
	pkt += [0x00]  # reserved byte
	pkt += [ID]
	pkt += [0x00, 0x00]  # length placeholder
	pkt += [instr]  # instruction
	if reg: pkt += le(reg)  # not everything has a register
	if params: pkt += params    # not everything has parameters

	length = le(len(pkt)-5)  # length = len(packet) - (header(3), reserve(1), id(1))
	pkt[5] = length[0]  # L
	pkt[6] = length[1]  # H

	crc = crc16(pkt)
	pkt += le(crc)

	return pkt


def makePingPacket(ID):
	"""
	Pings a servo
	"""
	#      | header          | rsv | id| length    |instr|
	# pkt = [0xff, 0xff, 0xfd, 0x00, ID, 0x03, 0x00, 0x01, crc_l, crc_h]
	pkt = makePacket(ID, xl320.XL320_PING)
	return pkt


def makeWritePacket(ID, reg, values=None):
	"""
	Creates a packet that writes a value(s) to servo ID at location reg. Make
	sure the values are in little endian (use Packet.le() if necessary) for 16 b
	(word size) values.
	"""
	pkt = makePacket(ID, xl320.XL320_WRITE, reg, values)
	return pkt


def makeReadPacket(ID, reg, values=None):
	"""
	Creates a packet that reads the register(s) of servo ID at location reg. Make
	sure the values are in little endian (use Packet.le() if necessary) for 16 b
	(word size) values.
	"""
	pkt = makePacket(ID, xl320.XL320_READ, reg, values)
	return pkt


def makeResetPacket(ID, param):
	"""
	Resets a servo to one of 3 reset states:

	XL320_RESET_ALL                  = 0xFF
	XL320_RESET_ALL_BUT_ID           = 0x01
	XL320_RESET_ALL_BUT_ID_BAUD_RATE = 0x02
	"""
	if param not in [0x01, 0x02, 0xff]:
		raise Exception('Packet.makeResetPacket invalide parameter {}'.format(param))
	# pkt = makePacket(ID, xl320.XL320_RESET, None, [param])
	pkt = makePacket(ID, xl320.XL320_RESET, None, [1])
	return pkt


def makeRebootPacket(ID):
	"""
	Reboots a servo
	"""
	pkt = makePacket(ID, xl320.XL320_REBOOT)
	return pkt


def makeServoIDPacket(curr_id, new_id):
	"""
	Given the current ID, returns a packet to set the servo to a new ID
	"""
	pkt = makeWritePacket(curr_id, xl320.XL320_ID, [new_id])
	return pkt


# [0xFF, 0xFF, 0xFD, 0x00, ID, LEN_L, LEN_H, 0x30, ANGLE_L, ANGLE_H, CRC_L, CRC_H]
def makeServoPacket(ID, angle):
	"""
	Commands the servo to an angle (in degrees)
	"""
	if not (0.0 <= angle <= 300.0):
		raise Exception('makeServoPacket(), angle out of bounds: {}'.format(angle))
	val = int(angle/300*1023)
	# lo, hi = le(val)
	# print('servo cmd {} deg : {} : L {} H {}'.format(angle, val, lo, hi))
	pkt = makeWritePacket(ID, xl320.XL320_GOAL_POSITION, le(val))
	return pkt


def makeServoMaxLimitPacket(ID, angle):
	"""
	Sets the maximum servo angle (in the CCW direction)
	"""
	angle = int(angle/300.0*1023)
	pkt = makeWritePacket(ID, xl320.XL320_CCW_ANGLE_LIMIT, le(angle))
	return pkt


def makeServoMinLimitPacket(ID, angle):
	"""
	Sets the minimum servo angle (in the CW direction)
	"""
	angle = int(angle/300.0*1023)
	pkt = makeWritePacket(ID, xl320.XL320_CW_ANGLE_LIMIT, le(angle))
	return pkt


def makeServoSpeedPacket(ID, maxSpeed):
	"""
	Run servo between 0.0 to 1.0, where 1.0 is 100% (full) speed.
	"""
	if 0.0 > maxSpeed > 1.0:
		raise Exception('makeServoSpeed: max speed is a percentage (0.0-1.0)')
	speed = int(maxSpeed*1023)
	pkt = makeWritePacket(ID, xl320.XL320_GOAL_VELOCITY, le(speed))
	return pkt


def makeLEDPacket(ID, color):
	"""
	Turn on/off the servo LED and also sets the color.
	"""
	# pkt = [255, 255, 253, 0, ID, 11, 0, 3, 25, 0, 2 crc_l, crc_h]
	pkt = makeWritePacket(ID, xl320.XL320_LED, [color])
	return pkt


def makeDelayPacket(ID, delay):
	"""
	It is the delay time per data value that takes from the transmission of
	Instruction Packet until the return of Status Packet.
	0 to 254 (0xFE) can be used, and the delay time per data value is 2 usec.
	That is to say, if the data value is 10, 20 usec is delayed. The initial
	value is 250 (0xFA) (i.e., 0.5 msec).
	"""
	pkt = makeWritePacket(ID, xl320.XL320_DELAY_TIME, delay)
	return pkt


def makeControlModePacket(ID, mode):
	"""
	Sets the xl-320 to either servo or wheel mode
	"""
	pkt = makeWritePacket(ID, xl320.XL320_CONTROL_MODE, le(mode))
	return pkt


def makeBaudRatePacket(ID, rate):
	"""
	Set baud rate of servo.

	in: rate - 0: 9600, 1:57600, 2:115200, 3:1Mbps
	out: write packet
	"""
	if rate not in [0, 1, 2, 3]:
		raise Exception('Packet.makeBaudRatePacket: wrong rate {}'.format(rate))
	pkt = makeWritePacket(ID, xl320.XL320_BAUD_RATE, [rate])
	return pkt


def makeSyncAnglePacket(info):
	"""
	Write sync angle information to servos.

	info = [[ID, angle], [ID, angle], ...]
	"""
	addr = le(xl320.XL320_GOAL_POSITION)
	data = []

	# since all servo angles have the same register addr (XL320_GOAL_POSITION)
	# and data size (2), a sinc packet is smart choice
	# compare bulk vs sync for the same commands:
	# bulk = 94 bytes
	# sync = 50 bytes
	data.append(addr[0])  # LSB
	data.append(addr[1])  # MSB
	data.append(2)  # data size LSM
	data.append(0)  # data size MSB
	for pkt in info:
		data.append(pkt[0])  # ID
		angle = le(int(pkt[1]/300*1023))
		data.append(angle[0])  # LSB
		data.append(angle[1])  # MSB

	ID = xl320.XL320_BROADCAST_ADDR
	instr = xl320.XL320_SYNC_WRITE
	pkt = makePacket(ID, instr, None, data)  # create packet

	# print(pkt)

	return pkt


def makeBulkAnglePacket(info):
	"""
	Write bulk angle information to servos.

	info = [[ID, angle], [ID, angle], ...]
	"""
	addr = le(xl320.XL320_GOAL_POSITION)
	data = []
	for pkt in info:
		data.append(pkt[0])  # ID
		data.append(addr[0])  # LSB
		data.append(addr[1])  # MSB
		data.append(2)  # 2 bytes
		data.append(0)
		angle = le(int(pkt[1]/300*1023))
		data.append(angle[0])  # LSB
		data.append(angle[1])  # MSB

	ID = xl320.XL320_BROADCAST_ADDR
	instr = xl320.XL320_BULK_WRITE
	pkt = makePacket(ID, instr, None, data)  # create packet

	return pkt


def getPacketType(pkt):
	"""
	Returns the packet type:

	XL320_PING   = 0x01
	XL320_READ   = 0x02
	XL320_WRITE  = 0x03
	XL320_RESET  = 0x06
	XL320_REBOOT = 0x08
	XL320_STATUS = 0x55
	"""
	return pkt[7]


# def getErrorString(pkt):
# 	"""
# 	not done - still in work, not sure I like this
#
# 	if error - return error_num, string
# 	if not error - returns 0, None
# 	"""
# 	# bit 7 - alert
# 	# bit 0-6 - error number 0-64
# 	# err_str = {
# 	# 	0: 'none',
# 	# 	1: 'result fail',
# 	# 	2: 'instruction error',
# 	# 	4: 'crc error',
# 	# 	8: 'data range error',
# 	# 	16: 'data length error',
# 	# 	32: '?',
# 	# 	64: '?'
# 	# }
# # 	err_str = [
# # 		'none',
# # 		'result fail',
# # 		'instruction error',
# # 		'crc error',
# # 		'data range error',
# # 		'data length error',
# # 		'?',
# # 		'?'
# # 	]
# 	ret = None
# 	err = 0
#
# 	if len(pkt) >= 11 and pkt[7] == xl320.XL320_STATUS:
# 			# err = pkt[8] | 128  # remove alert bit
# 			err = pkt[8]
# 			# ret = err_str[err]
# 			ret = 'error: {}'.format(err)
# 	return err, ret


def statusError(pkt):
	"""
	Grabs the status (int) from an error packet and returns it. It retuns -1
	if the packet is not a status packet.
	"""
	if pkt[7] == 0x55:
		return pkt[8]
	return -1


def getErrorString(pkt):
	"""
	Decodes a status packet to a string.
	"""
	err = statusError(pkt)
	return xl320.ErrorStatusMsg[err]


def prettyPrintPacket(pkt):
	"""
	not done
	"""
	s = 'packet ID: {} instr: {} len: {}'.format(pkt[4], pkt[7], int((pkt[6] << 8) + pkt[5]))
	if len(s) > 10:
		params = pkt[8:-2]
		s += ' params: {}'.format(params)
	return s


def findPkt(pkt):
	"""
	Search through a string of binary for a valid xl320 package.

	in: buffer to search through
	out: a list of valid data packet
	"""
	# print('findpkt', pkt)
	# print('-----------------------')
	ret = []
	while len(pkt)-10 >= 0:
		if pkt[0:4] != [0xFF, 0xFF, 0xFD, 0x00]:
			pkt.pop(0)  # get rid of the first index
			# print(' - pop:', pkt)
			continue
		# print(' > good packet')
		length = (pkt[6] << 8) + pkt[5]
		# print(' > length', length)
		crc_pos = 5 + length
		pkt_crc = pkt[crc_pos:crc_pos + 2]
		crc = le(crc16(pkt[:crc_pos]))
		# print(' > calc crc', crc)
		# print(' > pkt crc', pkt_crc)
		if pkt_crc == crc:
			pkt_end = crc_pos+2
			ret.append(pkt[:pkt_end])
			# print(' > found:', pkt[:pkt_end])
			# print(' > pkt size', pkt_end)
			del pkt[:pkt_end]
			# print(' > remaining:', pkt)
		else:
			pkt_end = crc_pos+2
			# print(' - crap:', pkt[:pkt_end])
			del pkt[:pkt_end]
	# print('findpkt ret:', ret)
	return ret


def packetToDict(pkt):
	"""
	Given a packet, this turns it into a dictionary ... is this useful?

	in: packet, array of numbers
	out: dictionary (key, value)
	"""

	d = {
		'id': pkt[4],
		'instruction': xl320.InstrToStr[pkt[7]],
		'length': (pkt[6] << 8) + pkt[5],
		'params': pkt[8:-2],
		'crc': pkt[-2:]
	}

	return d



#!/usr/bin/env python

##############################################
# The MIT License (MIT)
# Copyright (c) 2016 Kevin Walchko
# see LICENSE for full details
##############################################
# ----------------------------
# Simple tool to change the id number of a servo
#
# I believe I am doing this correct, but it doesn't work

from __future__ import print_function, division
from pyxl320 import ServoSerial
from pyxl320 import Packet
from pyxl320 import xl320
# from pyxl320 import DummySerial
from pyxl320 import utils
from time import sleep
import argparse
import sys
PY3 = sys.version_info > (3,)


def get_input(s):
	"""Handle difference between py2 and py3"""
	if PY3:
		return input(s)
	else:
		return raw_input(s)


def handleArgs():
	parser = argparse.ArgumentParser(description='Resets servo(s) to factory defaults')
	parser.add_argument('-a', '--all', help='reset all servos to defaults', action='store_true')
	parser.add_argument('-i', '--id', help='servo id', type=int, default=1)
	parser.add_argument('-l', '--level', help='reset level: 0-reset all, 1-reset all but ID, 2-reset all but ID and baud rate', type=int, default=2)
	# parser.add_argument('-c', '--current_id', help='current id', type=int, default=1)
	parser.add_argument('-p', '--port', help='serial port or \'dummy\' for testing, default is \'/dev/serial0\'', type=str, default='/dev/serial0')
	parser.add_argument('-y', '--yes', help='answer "yes" to prompt and force reset', action='store_true')

	args = vars(parser.parse_args())
	return args


if __name__ == '__main__':
	print('<<<< PyXL320 Servo Reset >>>>')
	args = handleArgs()

	if args['yes']:
		pass
	else:
		print('<<<<<<<<<< WARNING >>>>>>>>>>>>>')
		print(' You are about to reset your servo(s)')
		while True:
			ans = get_input(' Continue [y/n] >> ')
			if ans == 'y': break
			if ans == 'n': exit()
			print('please type "y" or "n"')

	port = args['port']

	if args['all']:
		ID = xl320.XL320_BROADCAST_ADDR
	else:
		ID = args['id']

	level = args['level']
	if level == 0:
		level = xl320.XL320_RESET_ALL
	elif level == 1:
		level = xl320.XL320_RESET_ALL_BUT_ID
	elif level == 2:
		level = xl320.XL320_RESET_ALL_BUT_ID_BAUD_RATE
	else:
		print('Sorry you selected an invalid reset level')
		exit()

	if port.lower() == 'dummy':
		ser = ServoSerial(port=port, fake=True)
	else:
		ser = ServoSerial(port=port)
	ser.open()

	pkt = Packet.makeResetPacket(ID, level)

	if args['all']:
		servo_str = 'all servos'
	else:
		servo_str = 'servo ID {}'.format(args['id'])

	if level == xl320.XL320_RESET_ALL:
		level_str = 'all parameters'
	elif level == xl320.XL320_RESET_ALL_BUT_ID:
		level_str = 'all parameters but ID'
	elif level == xl320.XL320_RESET_ALL_BUT_ID_BAUD_RATE:
		level_str = 'all parameters but ID and baudrate'

	print('\nResetting {}: {} set to default'.format(servo_str, level_str))
	ser.write(pkt)
	# print('packet', pkt)
	sleep(1)

	# do a reboot -----------------------------------------------------------
	pkt = Packet.makeRebootPacket(ID)
	ser.write(pkt)
	print('\nYour servo(s) should be reset now')
	sleep(1)

	# maybe include smarter logic than just spamming every servo ------------
	pkt = Packet.makePingPacket(xl320.XL320_BROADCAST_ADDR)
	ser.write(pkt)
	for cnt in range(3):
		ans = ser.read()

		if ans:
			for pkt in ans:
				servo = Packet.packetToDict(pkt)
				utils.prettyPrintPacket(servo)
				print('raw pkt: {}'.format(pkt))
		else:
			print('Try {}: no servos found'.format(cnt))

		sleep(0.1)



#
# from __future__ import division
# from __future__ import print_function
#
#
# class DummySerial(object):
# 	"""
# 	A dummy interface to test with when not hooked up to real hardware. It does
# 	a decent job of mimicing the real thing.
# 	"""
# 	def __init__(self, port=0, rate=0):
# 		print('WARNING: Using fake ServoSerial')
#
# 	def open(self):
# 		pass
#
# 	def setRTS(self):
# 		pass
#
# 	def sendPkt(self, pkt):
# 		return self.read()
#
# 	def read(self, how_much=128):
# 		return [
# 			[0xFF, 0xFF, 0xFD, 0x00, 0x01, 0x04, 0x00, 0x55, 0x00, 0xA1, 0x0C],
# 			[0xFF, 0xFF, 0xFD, 0x00, 0x03, 0x04, 0x00, 0x55, 0x00, 0xA1, 0x0C]
# 		]
#
# 	def write(self, data):
# 		return len(data)
#
# 	def close(self):
# 		pass
#
# 	def flushInput(self):
# 		pass



#!/usr/bin/env python

##############################################
# The MIT License (MIT)
# Copyright (c) 2016 Kevin Walchko
# see LICENSE for full details
##############################################

from pyxl320 import Packet
# from pyxl320 import DummySerial
from pyxl320 import ServoSerial
import argparse


def handleArgs():
	parser = argparse.ArgumentParser(description='Sets a servo to an angle')
	parser.add_argument('-i', '--id', help='servo id', type=int, default=1)
	parser.add_argument('angle', help='servo angle in degrees: 0.0 - 300.0', type=float)
	parser.add_argument('-p', '--port', help='serial port  or \'dummy\' for testing, default is \'/dev/serial0\'', type=str, default='/dev/serial0')

	args = vars(parser.parse_args())
	return args


def main():
	args = handleArgs()

	ID = args['id']
	port = args['port']  # '/dev/tty.usbserial-A5004Flb'
	angle = args['angle']

	print('Setting servo[{}] to {:.2f} on port {}'.format(ID, angle, port))

	if port.lower() == 'dummy':
		serial = ServoSerial(port=port, fake=True)
	else:
		serial = ServoSerial(port=port)
	serial.open()

	pkt = Packet.makeServoPacket(ID, angle)  # move servo 1 to 158.6 degrees
	err_no, err_str = serial.sendPkt(pkt)  # send packet to servo
	if err_no:
		print('Oops ... something went wrong!: {}'.format(err_str))


if __name__ == '__main__':
	main()



##############################################
# The MIT License (MIT)
# Copyright (c) 2016 Kevin Walchko
# see LICENSE for full details
##############################################

"""
This information is specific to Dynamixel's XL-320 servo motor.
http://support.robotis.com/en/product/dynamixel/xl-series/xl-320.htm
"""
# --------- INSTRUCTIONS -----
XL320_PING      = 0x01
XL320_READ      = 0x02
XL320_WRITE     = 0x03
XL320_REG_WRITE = 0x04
XL320_ACTION    = 0x05
XL320_RESET     = 0x06
XL320_REBOOT    = 0x08
XL320_STATUS    = 0x55
XL320_SYNC_READ  = 0x82
XL320_SYNC_WRITE = 0x83
XL320_BULK_READ  = 0x92
XL320_BULK_WRITE = 0x93

# -------- EEPROM -------------
XL320_MODEL_NUMBER    = 0
XL320_VER_FIRMWARE    = 2
XL320_ID              = 3
XL320_BAUD_RATE       = 4
XL320_DELAY_TIME      = 5
XL320_CW_ANGLE_LIMIT  = 6   # min angle, default 0
XL320_CCW_ANGLE_LIMIT = 8   # max angle, default 300
XL320_CONTROL_MODE    = 11  # joint or wheel mode, default joint (servo)
XL320_MAX_TORQUE      = 15
XL320_RETURN_LEVEL    = 17

# -------- RAM ----------------
XL320_TORQUE_ENABLE    = 24  # servo mode on/off - turn into wheel
XL320_LED              = 25
XL320_GOAL_POSITION    = 30
XL320_GOAL_VELOCITY    = 32
XL320_GOAL_TORQUE      = 35
XL320_PRESENT_POSITION = 37  # current servo angle
XL320_PRESENT_SPEED    = 39  # current speed
XL320_PESENT_LOAD      = 41  # current load
XL320_PESENT_VOLTAGE   = 45  # current voltage
XL320_PESENT_TEMP      = 46  # current temperature
XL320_MOVING           = 49
XL320_HW_ERROR_STATUS  = 50
XL320_PUNCH            = 51

# --------- OTHER -------------
XL320_RESET_ALL                  = 0xFF
XL320_RESET_ALL_BUT_ID           = 0x01
XL320_RESET_ALL_BUT_ID_BAUD_RATE = 0x02
XL320_LED_WHITE                  = 7
XL320_LED_BLUE_GREEN             = 6
XL320_LED_PINK                   = 5
XL320_LED_BLUE                   = 4
XL320_LED_YELLOW                 = 3
XL320_LED_GREEN                  = 2
XL320_LED_RED                    = 1
XL320_LED_OFF                    = 0
XL320_BROADCAST_ADDR             = 0xfe  # a packet with this ID will go to all servos
XL320_WHEEL_MODE                 = 1
XL320_JOINT_MODE                 = 2  # normal servo
XL320_9600                       = 0  # 0: 9600, 1:57600, 2:115200, 3:1Mbps
XL320_57600                      = 1
XL320_115200                     = 2
XL320_1000000                    = 3

# Conversions from values to strings for debugging

Baudrates = {
	0: '9.6 kbps',
	1: '57.6 kbps',
	2: '115 kbps',
	3: '1 Mbps'
}

Modes = {
	1: 'Wheel',
	2: 'Joint'
}

ReturnTypes = {
	0: 'No Return',
	1: 'Only Read',
	2: 'Return All'
}

LEDColors = {
	0: 'Off',
	1: 'Red',
	2: 'Green',
	3: 'Yellow',
	4: 'Blue',
	5: 'Pink',
	6: 'Blue-Green',
	7: 'White'
}

Moving = {
	0: 'Completed',
	1: 'Moving'
}

ErrorMsg = {
	0: None,
	1: 'ERROR_OVERLOAD',
	2: 'ERROR_OVER_HEATING',
	4: 'ERROR_INPUT_VOLTAGE'
}

# I don't trust these error number, I bet the documentation is wrong
ErrorStatusMsg = {
	-1: 'Not a status packet',
	0: None,
	1: 'ERROR_RAIL_FAIL',
	2: 'ERROR_INTRUCTION',
	3: 'ERROR_CRC',
	4: 'ERROR_DATA_RANGE',
	5: 'ERROR_DATA_LENGTH',
	6: 'ERROR_DATA_LIMIT',
	7: 'ERROR_ACCEESS',
	128: 'ALERT'
}

InstrToStr = {
	1: 'Ping',
	2: 'Read',
	3: 'Write',
	6: 'Reset',
	8: 'Reboot',
	85: 'Status'
}



#!/usr/bin/env python

##############################################
# The MIT License (MIT)
# Copyright (c) 2016 Kevin Walchko
# see LICENSE for full details
##############################################
# ----------------------------
# Simple tool to change the id number of a servo
#

from __future__ import print_function, division
from pyxl320 import ServoSerial, Packet, xl320
# from pyxl320 import DummySerial
import argparse
import sys
PY3 = sys.version_info > (3,)


def get_input(s):
	"""Handle difference between py2 and py3"""
	if PY3:
		return input(s)
	else:
		return raw_input(s)


def makeServoIDPacket(curr_id, new_id):
	"""
	Given the current ID, returns a packet to set the servo to a new ID
	"""
	pkt = Packet.makeWritePacket(curr_id, xl320.XL320_ID, [new_id])
	return pkt


def makeInterActive():
	port = get_input('Enter serial port >> ')
	curr_id = get_input('Enter current id >> ')
	new_id = get_input('Enter new id >> ')
	return port, curr_id, new_id


def handleArgs():
	parser = argparse.ArgumentParser(description='set servo id')
	parser.add_argument('-i', '--interactive', help='input via commandline', action='store_true')
	parser.add_argument('-n', '--new_id', help='set new id', type=int, default=1)
	parser.add_argument('-c', '--current_id', help='current id', type=int, default=1)
	parser.add_argument('-p', '--port', help='serial port', type=str, default='/dev/tty.usbserial-A5004Flb')

	args = vars(parser.parse_args())
	return args


def main():
	args = handleArgs()

	if args['interactive']:
		port, curr_id, new_id = makeInterActive()
	else:
		port = args['port']
		curr_id = args['current_id']
		new_id = args['new_id']

	if port == 'dummy':
		ser = ServoSerial(port=port, fake=True)
	else:
		ser = ServoSerial(port=port)
	ser.open()

	pkt = makeServoIDPacket(curr_id, new_id)
	print('pkt', pkt)

	err, err_str = ser.sendPkt(pkt)
	if err:
		print('Error: {} {}'.format(err, err_str))

if __name__ == '__main__':
	main()





##############################################
# The MIT License (MIT)
# Copyright (c) 2016 Kevin Walchko
# see LICENSE for full details
##############################################
# Serial interfaces (real and test) for communications with XL-320 servos.

from __future__ import division
from __future__ import print_function
import serial as PySerial
from . import Packet
from fake_rpi import serial as FakeSerial
import time


# class FakeSerial(object):
#
# 	class Serial(object):
# 		"""
# 		A dummy interface to test with when not hooked up to real hardware.
# 		"""
# 		cd = False
# 		cts = True
# 		dsr = False
# 		in_waiting = True
# 		out_waiting = False
# 		ri = False
# 		port = 'port'
# 		baudrate = 0
# 		timeout = 0
# 		status = False  # open or closed
# 		name = 'name'
#
# 		def __init__(self):
# 			pass
#
# 		def open(self):
# 			self.status = True
#
# 		def isOpen(self):
# 			return self.status
#
# 		def setRTS(self, a):
# 			pass
#
# 		def read(self, size=1):
# 			pass
#
# 		def write(self, data):
# 			return len(data)
#
# 		def close(self):
# 			self.status = False
#
# 		def flush(self):
# 			pass
#
# 		def set_output_flow_control(self, enable=True):
# 			pass
#
# 		def set_input_flow_control(self, enable=True):
# 			pass
#
# 		def cancel_read(self):
# 			pass
#
# 		def cancel_write(self):
# 			pass
#
# 		def get_settings(self):
# 			return {'port': self.port}


class ServoSerial(object):
	"""
	A wrapper around pyserial to work with Dynamixel servos' half duplex
	interface. This requires extra hardware added to your normal full duplex
	serial port. Also, this uses the  RTS pin to toggle between Tx and Rx.

	All data that goes into this class via write() or returns from it via read()
	is a simple array of bytes (e.g., [1,34,234,1,0,24,67]). Internally, the class
	transforms those into a binary stream.

	This class also uses Packet to find and verify what is returned form read()
	is a valid packet.

	RPi3 sucks ... they screwed up the serial port, the RTS pin doesn't work so I
	just toggle pin 17 and treat it as an output pin.
	"""
# 	DD_WRITE = False      # data direction set to write .. RTS is backwards
# 	DD_READ = True        # data direction set to read .. RTS is backwards
	DD_WRITE = True      # data direction set to write
	DD_READ = False        # data direction set to read
	# SLEEP_TIME = 0.0    # sleep time between read/write
	# SLEEP_TIME = 0.005    # sleep time between read/write
	# SLEEP_TIME = 0.0005    # sleep time between read/write
	SLEEP_TIME = 0.00005    # sleep time between read/write
	# SLEEP_TIME = 0.000005    # sleep time between read/write

	def __init__(self, port, baud_rate=1000000, fake=False):
		"""
		Constructor: sets up the serial port

		If you want to use a USB serial port, then set rts_hw to 0 (False). If you
		want to use the RPi seiral port, then you need to use a pin to toggle TX/Rx.
		Set rst_hw to any valid BCM pin greater than 0.
		"""
		if fake:
			self.serial = FakeSerial.Serial()
		else:
			self.serial = PySerial.Serial()
		self.serial.baudrate = baud_rate
		self.serial.port = port
		# the default time delay on the servo is 0.5 msec before it returns a status pkt
		self.serial.timeout = 0.0001  # time out waiting for blocking read()
		# self.serial.timeout = 0.0001

	def __del__(self):
		"""
		Destructor: closes the serial port
		"""
		self.close()

	def setRTS(self, level):
		time.sleep(self.SLEEP_TIME)
		self.serial.setRTS(not level)

	def open(self):
		if self.serial.isOpen():
			raise Exception('SeroSerial::open() ... Oops, port is already open')
		self.serial.open()
		self.setRTS(self.DD_WRITE)
		if self.serial.isOpen():
			print('Opened {} @ {}'.format(self.serial.name, self.serial.baudrate))
			print(self.serial.get_settings())
		else:
			raise Exception('Could not open {}'.format(self.serial.port))

	@staticmethod
	def decode(buff):
		"""
		Transforms the raw buffer data read in into a list of bytes
		"""
		pp = list(map(ord, buff))
		if 0 == len(pp) == 1:
			pp = []
		return pp

	def read(self, how_much=128):  # FIXME: 128 might be too much ... what is largest?
		"""
		This toggles the RTS pin and reads in data. It also converts the buffer
		back into a list of bytes and searches through the list to find valid
		packets of info. If there is more than one packet, this returns an
		array of valid packets.
		"""
		# ret = self.readPkts(how_much)
		# return ret
		ret = []
		self.setRTS(self.DD_READ)
		data = self.serial.read(how_much)
		# print('readPkts data', data)
		if data:
			data = self.decode(data)
			# print('decode', data)
			ret = Packet.findPkt(data)
			# print('ret', ret)
		return ret

	# def readPkts(self, how_much=128):  # FIXME: 128 might be too much ... what is largest?
	# 	"""
	# 	This toggles the RTS pin and reads in data. It also converts the buffer
	# 	back into a list of bytes and searches through the list to find valid
	# 	packets of info. If there is more than one packet, this returns an
	# 	array of valid packets.
	#
	# 	going to remove this one
	# 	"""
	# 	ret = []
	# 	self.setRTS(self.DD_READ)
	# 	data = self.serial.read(how_much)
	# 	# print('readPkts data', data)
	# 	if data:
	# 		data = self.decode(data)
	# 		# print('decode', data)
	# 		ret = Packet.findPkt(data)
	# 		# print('ret', ret)
	# 	return ret

	def write(self, pkt):
		"""
		This is a simple serial write command. It toggles the RTS pin and formats
		all of the data into bytes before it writes.
		"""
		self.setRTS(self.DD_WRITE)
		# prep data array for transmition
		pkt = bytearray(pkt)
		pkt = bytes(pkt)

		num = self.serial.write(pkt)
		# print('wrote {} of len(pkt) = {}'.format(num, len(pkt)))
		return num

	def sendPkt(self, pkt, retry=5, sleep_time=0.01):
		"""
		Sends a packet and waits for a return. If no return is given, then it
		resends the packet. If an error occurs, it also resends the packet.

		in:
			pkt - command packet to send to servo
			cnt - how many retries should this do? default = 5
		out:
			array of packets
		"""
		for cnt in range(retry):
			self.write(pkt)  # send packet to servo
			ans = self.read()  # get return status packet

			if ans:
				# check for error and resend
				return ans

			else:
				print('>> retry {} <<'.format(cnt))
				time.sleep(sleep_time)

		return []

	def close(self):
		"""
		If the serial port is open, it closes it.
		"""
		if self.serial.isOpen():
			self.serial.close()

	def flushInput(self):
		"""
		Flush the input.
		"""
		self.serial.flushInput()





##############################################
# The MIT License (MIT)
# Copyright (c) 2016 Kevin Walchko
# see LICENSE for full details
##############################################

import simplejson as json
from platform import python_version


if python_version().split('.')[0] == '2':
	import commands
else:
	import subprocess

	def getoutput(cmd):
		return subprocess.getoutput(cmd)


def listSerialPorts():
	"""
	http://pyserial.readthedocs.io/en/latest/shortintro.html

	This calls the command line tool from pyserial to list the available
	serial ports.
	"""
	cmd = 'python -m serial.tools.list_ports'
	err, ret = commands.getstatusoutput(cmd)
	if not err:
		r = ret.split('\n')
		ret = []
		for line in r:
			if line.find('/dev/') >= 0:
				line = line.replace(' ', '')
				ret.append(line)
	return err, ret


def hex_decode(data):
	"""
	Takes an array of number and turns them into a string of hex for printing.

	in: data - array of numbers [1,2,33,42,234]
	out: str - '0x1 0x02 0x21 0x2a 0xea'
	"""
	return ''.join(map('0x{:02X} '.format, data))


# def handleReturn(ans):
# 	if len(ans) > 0:
# 		if len(ans) == 1 and ans[0] == chr(0x00):
# 			print('No returned packet')
# 		else:
# 			print('Returned packet[{}]'.format(len(ans)))
# 			print('array:', hex_decode(ans))
# 	else:
# 		print('No returned packet')


def prettyPrintPacket(ctrl_table):
	"""
	This will pretty print out a packet's fields.

	in: dictionary of a packet
	out: nothing ... everything is printed to screen
	"""
	print('---------------------------------------')
	print("{:.<29} {}".format('id', ctrl_table['id']))
	ctrl_table.pop('id')
	for key, value in ctrl_table.items():
		print("{:.<29} {}".format(key, value))


class JsonFile(object):
	"""
	Simple class to read/write json files.
	"""
	@staticmethod
	def read(fname):
		"""
		Reads a Json file
		in: file name
		out: length of file, dictionary
		"""
		try:
			with open(fname, 'r') as f:
				data = json.load(f)
			return data

		except IOError:
			raise Exception('Could not open {0!s} for reading'.format((fname)))

	@staticmethod
	def write(fname, data):
		"""
		Writes a Json file

		in: fname - file name
			data - dictionary of data to put into the file

		out: nothing, everything is written to a file
		"""
		try:
			with open(fname, 'w') as f:
				json.dump(data, f)

		except IOError:
			raise Exception('Could not open {0!s} for writing'.format((fname)))





__version__ = '0.9.2'
__copyright__ = 'Copyright (c) 2016 Kevin Walchko'
__license__ = 'MIT'
__author__ = 'Kevin J. Walchko'


from .ServoSerial import ServoSerial
# from .DummySerial import DummySerial
from . import utils
# from utils import listSerialPorts
from . import xl320
from . import Packet



#!/usr/bin/env python

##############################################
# The MIT License (MIT)
# Copyright (c) 2016 Kevin Walchko
# see LICENSE for full details
##############################################

from __future__ import print_function, division
from pyxl320 import ServoSerial
from pyxl320 import Packet
# from pyxl320 import DummySerial
import argparse

rates = {
	9600: 0,
	57600: 1,
	115200: 2,
	1000000: 3
}


def handleArgs():
	parser = argparse.ArgumentParser(description='set servo buad rate')
	# parser.add_argument('-n', '--newid', help='set new id', type=int, default=1)
	parser.add_argument('-c', '--current_rate', help='current baud rate, default is 1000000', type=int, default=1000000)
	parser.add_argument('-i', '--id', help='current id', type=int, default=1)
	parser.add_argument('-p', '--port', help='serial port', type=str, default='/dev/tty.usbserial-A5004Flb')
	parser.add_argument('-r', '--new_rate', help='set new baud rate to 9600, 57600, 115200, or 1000000; default is 1000000', type=int, default=1000000)

	args = vars(parser.parse_args())
	return args


def main():
	args = handleArgs()

	port = args['port']
	ID = args['id']
	curr_rate = args['current_rate']
	# new_id = args['newid']
	# 0: 9600, 1:57600, 2:115200, 3:1Mbps
	new_rate = rates[args['new_rate']]

	if port == 'dummy':
		ser = ServoSerial(port=port, baud_rate=curr_rate, fake=True)
	else:
		ser = ServoSerial(port=port, baud_rate=curr_rate)
	ser.open()

	pkt = Packet.makeBaudRatePacket(ID, new_rate)
	ser.sendPkt(pkt)

	# pkt = Packet.makeRebootPacket(ID)
	# ser.sendPkt(pkt)

if __name__ == '__main__':
	main()
